@@ -107,6 +107,54 @@ int32_t SPVM__Net__SSLeay__alert_desc_string_long(SPVM_ENV* env, SPVM_VALUE* sta
   return 0;
 }
 
+int32_t SPVM__Net__SSLeay__load_client_CA_file(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t error_id = 0;
+  
+  void* obj_file = stack[0].oval;
+  
+  if (!obj_file) {
+    return env->die(env, stack, "The file $file must be defined.", __func__, FILE_NAME, __LINE__);
+  }
+  
+  const char* file = env->get_chars(env, stack, obj_file);
+  
+  STACK_OF(X509_NAME)* stack_of_x509_name = SSL_load_client_CA_file(file);
+  
+  if (!stack_of_x509_name) {
+    int64_t ssl_error = ERR_peek_last_error();
+    
+    char* ssl_error_string = env->get_stack_tmp_buffer(env, stack);
+    ERR_error_string_n(ssl_error, ssl_error_string, SPVM_NATIVE_C_STACK_TMP_BUFFER_SIZE);
+    
+    int32_t error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    
+    env->die(env, stack, "[OpenSSL Error]SSL_load_client_CA_file failed.", __func__, FILE_NAME, __LINE__);
+    return error_id;
+  }
+  
+  int32_t length = sk_X509_NAME_num(stack_of_x509_name);
+  void* obj_x509_names = env->new_object_array_by_name(env, stack, "Net::SSLeay::X509_NAME", length, &error_id, __func__, FILE_NAME, __LINE__);
+  
+  for (int32_t i = 0; i < length; i++) {
+    X509_NAME* x509_name = sk_X509_NAME_value(stack_of_x509_name, i);
+    
+    void* obj_address_x509_name = env->new_pointer_object_by_name(env, stack, "Address", x509_name, &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    stack[0].oval = obj_address_x509_name;
+    env->call_class_method_by_name(env, stack, "Net::SSLeay::X509_NAME", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    void* obj_x509_name = stack[0].oval;
+    
+    env->set_elem_object(env, stack, obj_x509_names, i, obj_x509_name);
+  }
+  
+  stack[0].oval = obj_x509_names;
+  
+  return 0;
+}
+
 // Instance Methods
 int32_t SPVM__Net__SSLeay__set_fd(SPVM_ENV* env, SPVM_VALUE* stack) {
   
@@ -552,8 +600,12 @@ int32_t SPVM__Net__SSLeay__set_SSL_CTX(SPVM_ENV* env, SPVM_VALUE* stack) {
     return error_id;
   }
   
-  void* obj_ret_ctx = env->new_pointer_object_by_name(env, stack, "Net::SSLeay::SSL_CTX", ret_ctx, &error_id, __func__, FILE_NAME, __LINE__);
+  void* obj_address_ret_ctx = env->new_pointer_object_by_name(env, stack, "Address", ret_ctx, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) { return error_id; }
+  stack[0].oval = obj_address_ret_ctx;
+  env->call_class_method_by_name(env, stack, "Net::SSLeay::SSL_CTX", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) { return error_id; }
+  void* obj_ret_ctx = stack[0].oval;
   
   env->set_field_object_by_name(env, stack, obj_self, "ssl_ctx", obj_ret_ctx, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) { return error_id; }
@@ -672,8 +724,12 @@ int32_t SPVM__Net__SSLeay__get_peer_certificate(SPVM_ENV* env, SPVM_VALUE* stack
   void* obj_x509 = NULL;
   
   if (x509) {
-    env->new_pointer_object_by_name(env, stack, "Net::SSLeay::X509", x509, &error_id, __func__, FILE_NAME, __LINE__);
+    void* obj_address_x509 = env->new_pointer_object_by_name(env, stack, "Address", x509, &error_id, __func__, FILE_NAME, __LINE__);
     if (error_id) { return error_id; }
+    stack[0].oval = obj_address_x509;
+    env->call_class_method_by_name(env, stack, "Net::SSLeay::X509", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    obj_x509 = stack[0].oval;
   }
   
   stack[0].oval = obj_x509;
@@ -724,8 +780,12 @@ int32_t SPVM__Net__SSLeay__get1_session(SPVM_ENV* env, SPVM_VALUE* stack) {
   void* obj_ssl_session = NULL;
   
   if (ssl_session) {
-    env->new_pointer_object_by_name(env, stack, "Net::SSLeay::SSL_SESSION", ssl_session, &error_id, __func__, FILE_NAME, __LINE__);
+    void* obj_address_ssl_session = env->new_pointer_object_by_name(env, stack, "Address", ssl_session, &error_id, __func__, FILE_NAME, __LINE__);
     if (error_id) { return error_id; }
+    stack[0].oval = obj_address_ssl_session;
+    env->call_class_method_by_name(env, stack, "Net::SSLeay::SSL_SESSION", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    obj_ssl_session = stack[0].oval;
   }
   
   stack[0].oval = obj_ssl_session;
@@ -782,10 +842,14 @@ int32_t SPVM__Net__SSLeay__get_certificate(SPVM_ENV* env, SPVM_VALUE* stack) {
   void* obj_x509 = NULL;
   
   if (x509) {
-    env->new_pointer_object_by_name(env, stack, "Net::SSLeay::X509", x509, &error_id, __func__, FILE_NAME, __LINE__);
+    void* obj_address_x509 = env->new_pointer_object_by_name(env, stack, "Address", x509, &error_id, __func__, FILE_NAME, __LINE__);
     if (error_id) { return error_id; }
+    stack[0].oval = obj_address_x509;
+    env->call_class_method_by_name(env, stack, "Net::SSLeay::X509", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    obj_x509 = stack[0].oval;
     
-    env->set_no_free(env, stack, x509, 1);
+    env->set_no_free(env, stack, obj_x509, 1);
   }
   
   stack[0].oval = obj_x509;
@@ -886,7 +950,14 @@ int32_t SPVM__Net__SSLeay__get_peer_cert_chain(SPVM_ENV* env, SPVM_VALUE* stack)
     for (int32_t i = 0; i < length; i++) {
       X509* x509 = sk_X509_value(stack_of_x509, i);
       X509_up_ref(x509);
-      void* obj_x509 = env->new_pointer_object_by_name(env, stack, "Net::SSLeay::X509", x509, &error_id, __func__, FILE_NAME, __LINE__);
+      
+      void* obj_address_x509 = env->new_pointer_object_by_name(env, stack, "Address", x509, &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      stack[0].oval = obj_address_x509;
+      env->call_class_method_by_name(env, stack, "Net::SSLeay::X509", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);
+      if (error_id) { return error_id; }
+      void* obj_x509 = stack[0].oval;
+      
       env->set_elem_object(env, stack, obj_x509s, i, obj_x509);
     }
   }
