@@ -913,10 +913,10 @@ int32_t SPVM__Net__SSLeay__SSL_CTX__DESTROY(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   void* obj_self = stack[0].oval;
   
-  SSL_CTX* ssl_ctx = env->get_pointer(env, stack, obj_self);
+  SSL_CTX* pointer = env->get_pointer(env, stack, obj_self);
   
   if (!env->no_free(env, stack, obj_self)) {
-    SSL_CTX_free(ssl_ctx);
+    SSL_CTX_free(pointer);
   }
   
   return 0;
